@@ -486,6 +486,7 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'min_id',
             'max_id',
             'timeout',
+            'force_timeout',
             'try_inotify',
             'command',
             'enumerate',
@@ -510,6 +511,7 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'override_homedir',
             'fallback_homedir',
             'override_shell',
+            'default_shell',
             'pwd_expiration_warning',
             'id_provider',
             'auth_provider',
@@ -717,7 +719,7 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'local': ['id', 'auth', 'chpass'],
             'ldap': ['id', 'auth', 'access', 'chpass', 'sudo', 'autofs'],
             'krb5': ['auth', 'access', 'chpass'],
-            'proxy': ['id', 'auth'],
+            'proxy': ['id', 'auth', 'chpass'],
             'simple': ['access'],
             'permit': ['access'],
             'deny': ['access']}
@@ -824,6 +826,7 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'min_id',
             'max_id',
             'timeout',
+            'force_timeout',
             'try_inotify',
             'command',
             'enumerate',
@@ -848,6 +851,7 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'override_homedir',
             'fallback_homedir',
             'override_shell',
+            'default_shell',
             'pwd_expiration_warning',
             'id_provider',
             'auth_provider',
