@@ -39,6 +39,8 @@ struct ipa_access_ctx {
     time_t last_update;
     struct sdap_access_ctx *sdap_access_ctx;
 
+    struct sdap_attr_map *host_map;
+    struct sdap_attr_map *hostgroup_map;
     struct sdap_search_base **host_search_bases;
     struct sdap_search_base **hbac_search_bases;
 };
@@ -55,7 +57,6 @@ struct hbac_ctx {
     struct sdap_search_base **search_bases;
 
     /* Hosts */
-    const char **host_attrs;
     size_t host_count;
     struct sysdb_attrs **hosts;
     size_t hostgroup_count;
@@ -84,8 +85,8 @@ static inline struct be_ctx *hbac_ctx_be(struct hbac_ctx *hbac_ctx)
 /* Get sysdb associated with HBAC context */
 static inline struct sysdb_ctx *hbac_ctx_sysdb(struct hbac_ctx *hbac_ctx)
 {
-    struct be_ctx *be_ctx = hbac_ctx_be(hbac_ctx);
-    return be_ctx != NULL ? be_ctx->sysdb : NULL;
+    struct be_req *be_req = hbac_ctx->be_req;
+    return be_req != NULL ? be_req->sysdb : NULL;
 }
 
 /* Get tevent context associated with HBAC context */
@@ -116,4 +117,9 @@ static inline bool hbac_ctx_is_offline(struct hbac_ctx *ctx)
 
 void ipa_access_handler(struct be_req *be_req);
 
+errno_t hbac_get_cached_rules(TALLOC_CTX *mem_ctx,
+                              struct sysdb_ctx *sysdb,
+                              size_t *_rule_count,
+                              struct sysdb_attrs ***_rules);
+
 #endif /* _IPA_ACCESS_H_ */
