@@ -32,35 +32,24 @@ struct ipa_service {
     struct krb5_service *krb5_service;
 };
 
-/* the following defines are used to keep track of the options in the ldap
- * module, so that if they change and ipa is not updated correspondingly
- * this will trigger a runtime abort error */
-#define IPA_OPTS_BASIC_TEST 61
-
-#define IPA_OPTS_SVC_TEST 5
-
-#define IPA_OPTS_AUTOMNTMAP_TEST 2
-#define IPA_OPTS_AUTOMNTENTRY_TEST 3
-
-/* the following define is used to keep track of the options in the krb5
- * module, so that if they change and ipa is not updated correspondingly
- * this will trigger a runtime abort error */
-#define IPA_KRB5_OPTS_TEST 15
-
 enum ipa_basic_opt {
     IPA_DOMAIN = 0,
     IPA_SERVER,
+    IPA_BACKUP_SERVER,
     IPA_HOSTNAME,
     IPA_DYNDNS_UPDATE,
     IPA_DYNDNS_IFACE,
     IPA_HBAC_SEARCH_BASE,
     IPA_HOST_SEARCH_BASE,
     IPA_SELINUX_SEARCH_BASE,
+    IPA_SUBDOMAINS_SEARCH_BASE,
+    IPA_MASTER_DOMAIN_SEARCH_BASE,
     IPA_KRB5_REALM,
     IPA_HBAC_REFRESH,
     IPA_HBAC_DENY_METHOD,
     IPA_HBAC_SUPPORT_SRCHOST,
     IPA_AUTOMOUNT_LOCATION,
+    IPA_RANGES_SEARCH_BASE,
 
     IPA_OPTS_BASIC /* opts counter */
 };
@@ -81,13 +70,26 @@ enum ipa_netgroup_attrs {
 
 enum ipa_host_attrs {
     IPA_OC_HOST = 0,
+    IPA_AT_HOST_NAME,
     IPA_AT_HOST_FQDN,
+    IPA_AT_HOST_SERVERHOSTNAME,
     IPA_AT_HOST_MEMBER_OF,
     IPA_AT_HOST_SSH_PUBLIC_KEY,
+    IPA_AT_HOST_UUID,
 
     IPA_OPTS_HOST /* attrs counter */
 };
 
+enum ipa_hostgroup_attrs {
+    IPA_OC_HOSTGROUP = 0,
+    IPA_AT_HOSTGROUP_NAME,
+    IPA_AT_HOSTGROUP_MEMBER,
+    IPA_AT_HOSTGROUP_MEMBER_OF,
+    IPA_AT_HOSTGROUP_UUID,
+
+    IPA_OPTS_HOSTGROUP /* attrs counter */
+};
+
 enum ipa_selinux_usermap_attrs {
     IPA_OC_SELINUX_USERMAP = 0,
     IPA_AT_SELINUX_USERMAP_NAME,
@@ -118,9 +120,16 @@ struct ipa_id_ctx {
 struct ipa_options {
     struct dp_option *basic;
 
+    struct sdap_attr_map *host_map;
+    struct sdap_attr_map *hostgroup_map;
+    struct sdap_attr_map *selinuxuser_map;
+
     struct sdap_search_base **host_search_bases;
     struct sdap_search_base **hbac_search_bases;
     struct sdap_search_base **selinux_search_bases;
+    struct sdap_search_base **subdomains_search_bases;
+    struct sdap_search_base **master_domain_search_bases;
+    struct sdap_search_base **ranges_search_bases;
     struct ipa_service *service;
 
     /* id provider */
@@ -163,7 +172,8 @@ int ipa_autofs_init(struct be_ctx *be_ctx,
                     void **pvt_data);
 
 int ipa_service_init(TALLOC_CTX *memctx, struct be_ctx *ctx,
-                     const char *servers,
+                     const char *primary_servers,
+                     const char *backup_servers,
                      struct ipa_options *options,
                      struct ipa_service **_service);
 
