@@ -61,7 +61,7 @@ ipa_get_config_send(TALLOC_CTX *mem_ctx,
             goto done;
         }
         state->attrs[0] = IPA_CONFIG_MIGRATION_ENABLED;
-        state->attrs[1] = IPA_CONFIG_SELINUX_DEFAULT_MAP;
+        state->attrs[1] = IPA_CONFIG_SELINUX_DEFAULT_USER_CTX;
         state->attrs[2] = IPA_CONFIG_SELINUX_MAP_ORDER;
         state->attrs[3] = NULL;
     } else {
