@@ -523,12 +523,12 @@ static int ipa_netgr_fetch_hosts(struct ipa_get_netgroups_state *state,
     filter = talloc_asprintf(state, "(&%s%s(objectclass=%s))",
                              state->filter,
                              base_filter?base_filter:"",
-                             state->opts->host_map[IPA_OC_HOST].name);
+                             state->ipa_opts->host_map[IPA_OC_HOST].name);
     if (filter == NULL)
         return ENOMEM;
 
-    ret = build_attrs_from_map(state, state->opts->host_map,
-                               IPA_OPTS_HOST, &attrs);
+    ret = build_attrs_from_map(state, state->ipa_opts->host_map,
+                               IPA_OPTS_HOST, NULL, &attrs, NULL);
     if (ret != EOK) {
         talloc_free(filter);
         return ret;
@@ -537,7 +537,7 @@ static int ipa_netgr_fetch_hosts(struct ipa_get_netgroups_state *state,
     subreq = sdap_get_generic_send(state, state->ev, state->opts, state->sh,
                                    bases[state->host_base_iter]->basedn,
                                    bases[state->host_base_iter]->scope,
-                                   filter, attrs, state->opts->host_map,
+                                   filter, attrs, state->ipa_opts->host_map,
                                    IPA_OPTS_HOST, state->timeout, true);
 
     state->current_entity = ENTITY_HOST;
@@ -712,7 +712,7 @@ static bool extract_entities(hash_entry_t *entry, void *pvt)
     state = talloc_get_type(pvt, struct extract_state);
     member = talloc_get_type(entry->value.ptr, struct sysdb_attrs);
 
-    ret = sysdb_attrs_get_el(member, SYSDB_MEMBEROF, &el);
+    ret = sysdb_attrs_get_el(member, SYSDB_ORIG_MEMBEROF, &el);
     if (ret != EOK) return false;
 
     ret = sysdb_attrs_get_el(member, SYSDB_NAME, &name_el);
@@ -994,6 +994,7 @@ static int ipa_netgr_process_all(struct ipa_get_netgroups_state *state)
         }
     }
 
+    ret = EOK;
 done:
     return ret;
 }
