@@ -175,7 +175,12 @@ errno_t write_krb5info_file(const char *realm, const char *kdc,
 errno_t remove_krb5_info_files(TALLOC_CTX *mem_ctx, const char *realm);
 
 errno_t krb5_get_simple_upn(TALLOC_CTX *mem_ctx, struct krb5_ctx *krb5_ctx,
-                            const char *username, const char **_upn);
+                            const char *domain_name, const char *username,
+                            const char *user_dom, char **_upn);
+
+errno_t compare_principal_realm(const char *upn, const char *realm,
+                                bool *different_realm);
+
 
 int sssm_krb5_auth_init(struct be_ctx *bectx,
                         struct bet_ops **ops,
