@@ -242,7 +242,10 @@ static krb5_error_code ldap_child_get_tgt_sync(TALLOC_CTX *memctx,
 
         ret = select_principal_from_keytab(memctx, hostname, realm_name,
                 keytab_name, &full_princ, NULL, NULL);
-        if (ret) goto done;
+        if (ret) {
+            krberr = KRB5_KT_IOERR;
+            goto done;
+        }
     }
     if (!full_princ) {
         krberr = KRB5KRB_ERR_GENERIC;
