@@ -31,6 +31,7 @@
 #include "db/sysdb.h"
 #include "providers/ldap/ldap_common.h"
 #include "providers/ldap/sdap_async.h"
+#include "providers/ldap/sdap_idmap.h"
 
 /* =Users-Related-Functions-(by-name,by-uid)============================== */
 
@@ -65,7 +66,11 @@ struct tevent_req *users_get_send(TALLOC_CTX *memctx,
     struct users_get_state *state;
     const char *attr_name;
     char *clean_name;
+    char *endptr;
     int ret;
+    uid_t uid;
+    enum idmap_error_code err;
+    char *sid;
 
     req = tevent_req_create(memctx, &state, struct users_get_state);
     if (!req) return NULL;
@@ -89,20 +94,52 @@ struct tevent_req *users_get_send(TALLOC_CTX *memctx,
     switch (filter_type) {
     case BE_FILTER_NAME:
         attr_name = ctx->opts->user_map[SDAP_AT_USER_NAME].name;
+        ret = sss_filter_sanitize(state, name, &clean_name);
+        if (ret != EOK) {
+            goto fail;
+        }
         break;
     case BE_FILTER_IDNUM:
-        attr_name = ctx->opts->user_map[SDAP_AT_USER_UID].name;
+        if (dp_opt_get_bool(ctx->opts->basic, SDAP_ID_MAPPING)) {
+            /* If we're ID-mapping, we need to use the objectSID
+             * in the search filter.
+             */
+            uid = strtouint32(name, &endptr, 10);
+            if (errno != EOK) {
+                ret = EINVAL;
+                goto fail;
+            }
+
+            /* Convert the UID to its objectSID */
+            err = sss_idmap_unix_to_sid(ctx->opts->idmap_ctx->map,
+                                        uid, &sid);
+            if (err != IDMAP_SUCCESS) {
+                DEBUG(SSSDBG_MINOR_FAILURE,
+                      ("Mapping ID [%s] to SID failed: [%s]\n",
+                       name, idmap_error_string(err)));
+                ret = EIO;
+                goto fail;
+            }
+
+            attr_name = ctx->opts->user_map[SDAP_AT_USER_OBJECTSID].name;
+            ret = sss_filter_sanitize(state, sid, &clean_name);
+            if (ret != EOK) {
+                goto fail;
+            }
+
+        } else {
+            attr_name = ctx->opts->user_map[SDAP_AT_USER_UID].name;
+            ret = sss_filter_sanitize(state, name, &clean_name);
+            if (ret != EOK) {
+                goto fail;
+            }
+        }
         break;
     default:
         ret = EINVAL;
         goto fail;
     }
 
-    ret = sss_filter_sanitize(state, name, &clean_name);
-    if (ret != EOK) {
-        goto fail;
-    }
-
     state->filter = talloc_asprintf(state, "(&(%s=%s)(objectclass=%s))",
                                     attr_name, clean_name,
                                     ctx->opts->user_map[SDAP_OC_USER].name);
@@ -114,8 +151,8 @@ struct tevent_req *users_get_send(TALLOC_CTX *memctx,
     }
 
     /* TODO: handle attrs_type */
-    ret = build_attrs_from_map(state, ctx->opts->user_map,
-                               SDAP_OPTS_USER, &state->attrs);
+    ret = build_attrs_from_map(state, ctx->opts->user_map, SDAP_OPTS_USER,
+                               NULL, &state->attrs, NULL);
     if (ret != EOK) goto fail;
 
     ret = users_get_retry(req);
@@ -297,7 +334,12 @@ struct tevent_req *groups_get_send(TALLOC_CTX *memctx,
     struct groups_get_state *state;
     const char *attr_name;
     char *clean_name;
+    char *endptr;
     int ret;
+    gid_t gid;
+    enum idmap_error_code err;
+    char *sid;
+    bool use_id_mapping = dp_opt_get_bool(ctx->opts->basic, SDAP_ID_MAPPING);
 
     req = tevent_req_create(memctx, &state, struct groups_get_state);
     if (!req) return NULL;
@@ -321,27 +363,74 @@ struct tevent_req *groups_get_send(TALLOC_CTX *memctx,
     switch(filter_type) {
     case BE_FILTER_NAME:
         attr_name = ctx->opts->group_map[SDAP_AT_GROUP_NAME].name;
+
+        ret = sss_filter_sanitize(state, name, &clean_name);
+        if (ret != EOK) {
+            goto fail;
+        }
         break;
     case BE_FILTER_IDNUM:
-        attr_name = ctx->opts->group_map[SDAP_AT_GROUP_GID].name;
+        if (dp_opt_get_bool(ctx->opts->basic, SDAP_ID_MAPPING)) {
+            /* If we're ID-mapping, we need to use the objectSID
+             * in the search filter.
+             */
+            gid = strtouint32(name, &endptr, 10);
+            if (errno != EOK) {
+                ret = EINVAL;
+                goto fail;
+            }
+
+            /* Convert the UID to its objectSID */
+            err = sss_idmap_unix_to_sid(ctx->opts->idmap_ctx->map,
+                                        gid, &sid);
+            if (err != IDMAP_SUCCESS) {
+                DEBUG(SSSDBG_MINOR_FAILURE,
+                      ("Mapping ID [%s] to SID failed: [%s]\n",
+                       name, idmap_error_string(err)));
+                ret = EIO;
+                goto fail;
+            }
+
+            attr_name = ctx->opts->group_map[SDAP_AT_GROUP_OBJECTSID].name;
+            ret = sss_filter_sanitize(state, sid, &clean_name);
+            if (ret != EOK) {
+                goto fail;
+            }
+
+        } else {
+            attr_name = ctx->opts->group_map[SDAP_AT_GROUP_GID].name;
+            ret = sss_filter_sanitize(state, name, &clean_name);
+            if (ret != EOK) {
+                goto fail;
+            }
+        }
+        break;
         break;
     default:
         ret = EINVAL;
         goto fail;
     }
 
-    ret = sss_filter_sanitize(state, name, &clean_name);
-    if (ret != EOK) {
-        goto fail;
+    if (use_id_mapping) {
+        /* When mapping IDs, we don't want to limit ourselves
+         * to groups with a GID value
+         */
+
+        state->filter = talloc_asprintf(state,
+                                        "(&(%s=%s)(objectclass=%s)(%s=*))",
+                                        attr_name, clean_name,
+                                        ctx->opts->group_map[SDAP_OC_GROUP].name,
+                                        ctx->opts->group_map[SDAP_AT_GROUP_NAME].name);
+    } else {
+        state->filter = talloc_asprintf(state,
+                                        "(&(%s=%s)(objectclass=%s)(%s=*)(&(%s=*)(!(%s=0))))",
+                                        attr_name, clean_name,
+                                        ctx->opts->group_map[SDAP_OC_GROUP].name,
+                                        ctx->opts->group_map[SDAP_AT_GROUP_NAME].name,
+                                        ctx->opts->group_map[SDAP_AT_GROUP_GID].name,
+                                        ctx->opts->group_map[SDAP_AT_GROUP_GID].name);
     }
 
-    state->filter = talloc_asprintf(state,
-                                    "(&(%s=%s)(objectclass=%s)(%s=*)(&(%s=*)(!(%s=0))))",
-                                    attr_name, clean_name,
-                                    ctx->opts->group_map[SDAP_OC_GROUP].name,
-                                    ctx->opts->group_map[SDAP_AT_GROUP_NAME].name,
-                                    ctx->opts->group_map[SDAP_AT_GROUP_GID].name,
-                                    ctx->opts->group_map[SDAP_AT_GROUP_GID].name);
     talloc_zfree(clean_name);
     if (!state->filter) {
         DEBUG(2, ("Failed to build filter\n"));
@@ -350,8 +439,8 @@ struct tevent_req *groups_get_send(TALLOC_CTX *memctx,
     }
 
     /* TODO: handle attrs_type */
-    ret = build_attrs_from_map(state, ctx->opts->group_map,
-                               SDAP_OPTS_GROUP, &state->attrs);
+    ret = build_attrs_from_map(state, ctx->opts->group_map, SDAP_OPTS_GROUP,
+                               NULL, &state->attrs, NULL);
     if (ret != EOK) goto fail;
 
     ret = groups_get_retry(req);
@@ -542,8 +631,8 @@ static struct tevent_req *groups_by_user_send(TALLOC_CTX *memctx,
 
     state->name = name;
 
-    ret = build_attrs_from_map(state, ctx->opts->group_map,
-                               SDAP_OPTS_GROUP, &state->attrs);
+    ret = build_attrs_from_map(state, ctx->opts->group_map, SDAP_OPTS_GROUP,
+                               NULL, &state->attrs, NULL);
     if (ret != EOK) goto fail;
 
     ret = groups_by_user_retry(req);
@@ -708,6 +797,8 @@ fail:
     sdap_handler_done(be_req, DP_ERR_FATAL, ret, NULL);
 }
 
+static void sdap_check_online_reinit_done(struct tevent_req *req);
+
 static void sdap_check_online_done(struct tevent_req *req)
 {
     struct sdap_online_check_ctx *check_ctx = tevent_req_callback_data(req,
@@ -717,6 +808,9 @@ static void sdap_check_online_done(struct tevent_req *req)
     bool can_retry;
     struct sdap_server_opts *srv_opts;
     struct be_req *be_req;
+    struct sdap_id_ctx *id_ctx;
+    struct tevent_req *reinit_req = NULL;
+    bool reinit = false;
 
     ret = sdap_cli_connect_recv(req, NULL, &can_retry, NULL, &srv_opts);
     talloc_zfree(req);
@@ -732,23 +826,65 @@ static void sdap_check_online_done(struct tevent_req *req)
             srv_opts->max_user_value = 0;
             srv_opts->max_group_value = 0;
             srv_opts->max_service_value = 0;
+            srv_opts->max_sudo_value = 0;
         } else if (strcmp(srv_opts->server_id, check_ctx->id_ctx->srv_opts->server_id) == 0
                    && srv_opts->supports_usn
                    && check_ctx->id_ctx->srv_opts->last_usn > srv_opts->last_usn) {
             check_ctx->id_ctx->srv_opts->max_user_value = 0;
             check_ctx->id_ctx->srv_opts->max_group_value = 0;
             check_ctx->id_ctx->srv_opts->max_service_value = 0;
+            check_ctx->id_ctx->srv_opts->max_sudo_value = 0;
             check_ctx->id_ctx->srv_opts->last_usn = srv_opts->last_usn;
+
+            reinit = true;
         }
 
         sdap_steal_server_opts(check_ctx->id_ctx, &srv_opts);
     }
 
     be_req = check_ctx->be_req;
+    id_ctx = check_ctx->id_ctx;
     talloc_free(check_ctx);
+
+    if (reinit) {
+        DEBUG(SSSDBG_TRACE_FUNC, ("Server reinitialization detected. "
+                                  "Cleaning cache.\n"));
+        reinit_req = sdap_reinit_cleanup_send(be_req, be_req->be_ctx, id_ctx);
+        if (reinit_req == NULL) {
+            DEBUG(SSSDBG_CRIT_FAILURE, ("Unable to perform reinitialization "
+                                        "clean up.\n"));
+            /* not fatal */
+            goto done;
+        }
+
+        tevent_req_set_callback(reinit_req, sdap_check_online_reinit_done,
+                                be_req);
+        return;
+    }
+
+done:
     sdap_handler_done(be_req, dp_err, 0, NULL);
 }
 
+static void sdap_check_online_reinit_done(struct tevent_req *req)
+{
+    struct be_req *be_req = NULL;
+    errno_t ret;
+
+    be_req = tevent_req_callback_data(req, struct be_req);
+    ret = sdap_reinit_cleanup_recv(req);
+    talloc_zfree(req);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Unable to perform reinitialization "
+              "clean up [%d]: %s\n", ret, strerror(ret)));
+        /* not fatal */
+    } else {
+        DEBUG(SSSDBG_TRACE_FUNC, ("Reinitialization clean up completed\n"));
+    }
+
+    sdap_handler_done(be_req, DP_ERR_OK, 0, NULL);
+}
+
 /* =Get-Account-Info-Call================================================= */
 
 /* FIXME: embed this function in sssd_be and only call out
