@@ -59,7 +59,7 @@ struct sdap_access_ctx {
 struct tevent_req *
 sdap_access_send(TALLOC_CTX *mem_ctx,
                  struct tevent_context *ev,
-                 struct be_ctx *be_ctx,
+                 struct be_req *be_req,
                  struct sdap_access_ctx *access_ctx,
                  struct pam_data *pd);
 errno_t
