@@ -837,6 +837,7 @@ struct sdap_get_rootdse_state {
 };
 
 static void sdap_get_rootdse_done(struct tevent_req *subreq);
+static void sdap_get_matching_rule_done(struct tevent_req *subreq);
 
 struct tevent_req *sdap_get_rootdse_send(TALLOC_CTX *memctx,
                                          struct tevent_context *ev,
@@ -854,6 +855,7 @@ struct tevent_req *sdap_get_rootdse_send(TALLOC_CTX *memctx,
             "supportedFeatures",
             "supportedLDAPVersion",
             "supportedSASLMechanisms",
+            SDAP_ROOTDSE_ATTR_AD_VERSION,
             SDAP_ROOTDSE_ATTR_DEFAULT_NAMING_CONTEXT,
             SDAP_IPA_LAST_USN, SDAP_AD_LAST_USN,
             NULL
@@ -884,6 +886,11 @@ struct tevent_req *sdap_get_rootdse_send(TALLOC_CTX *memctx,
     return req;
 }
 
+/* This is not a real attribute, it's just there to avoid
+ * actually pulling real data down, to save bandwidth
+ */
+#define SDAP_MATCHING_RULE_TEST_ATTR "sssmatchingruletest"
+
 static void sdap_get_rootdse_done(struct tevent_req *subreq)
 {
     struct tevent_req *req = tevent_req_callback_data(subreq,
@@ -893,6 +900,8 @@ static void sdap_get_rootdse_done(struct tevent_req *subreq)
     struct sysdb_attrs **results;
     size_t num_results;
     int ret;
+    const char *filter;
+    const char *attrs[] = { SDAP_MATCHING_RULE_TEST_ATTR, NULL };
 
     ret = sdap_get_generic_recv(subreq, state, &num_results, &results);
     talloc_zfree(subreq);
@@ -918,7 +927,81 @@ static void sdap_get_rootdse_done(struct tevent_req *subreq)
     state->rootdse = talloc_steal(state, results[0]);
     talloc_zfree(results);
 
-    DEBUG(9, ("Got rootdse\n"));
+    DEBUG(SSSDBG_TRACE_INTERNAL, ("Got rootdse\n"));
+
+    /* Auto-detect the ldap matching rule if requested */
+    if ((!dp_opt_get_bool(state->opts->basic,
+                          SDAP_AD_MATCHING_RULE_INITGROUPS))
+            && !dp_opt_get_bool(state->opts->basic,
+                                SDAP_AD_MATCHING_RULE_GROUPS)) {
+        /* This feature is disabled for both groups
+         * and initgroups. Skip the auto-detection
+         * lookup.
+         */
+        DEBUG(SSSDBG_TRACE_INTERNAL,
+              ("Skipping auto-detection of match rule\n"));
+        tevent_req_done(req);
+        return;
+    }
+
+    DEBUG(SSSDBG_TRACE_INTERNAL,
+          ("Auto-detecting support for match rule\n"));
+
+    /* Create a filter using the matching rule. It need not point
+     * at any valid data. We're only going to be looking for the
+     * error code.
+     */
+    filter = "("SDAP_MATCHING_RULE_TEST_ATTR":"
+             SDAP_MATCHING_RULE_IN_CHAIN":=)";
+
+    /* Perform a trivial query with the matching rule in play.
+     * If it returns success, we know it is available. If it
+     * returns EIO, we know it isn't.
+     */
+    subreq = sdap_get_generic_send(state, state->ev, state->opts, state->sh,
+                                   "", LDAP_SCOPE_BASE, filter, attrs, NULL,
+                                   0, dp_opt_get_int(state->opts->basic,
+                                                     SDAP_SEARCH_TIMEOUT),
+                                   false);
+    if (!subreq) {
+        tevent_req_error(req, ENOMEM);
+        return;
+    }
+    tevent_req_set_callback(subreq, sdap_get_matching_rule_done, req);
+}
+
+static void sdap_get_matching_rule_done(struct tevent_req *subreq)
+{
+    errno_t ret;
+    struct tevent_req *req = tevent_req_callback_data(subreq,
+                                                      struct tevent_req);
+    struct sdap_get_rootdse_state *state = tevent_req_data(req,
+                                             struct sdap_get_rootdse_state);
+    size_t num_results;
+    struct sysdb_attrs **results;
+
+    ret = sdap_get_generic_recv(subreq, state, &num_results, &results);
+    talloc_zfree(subreq);
+    if (ret == EOK) {
+        /* The search succeeded */
+        state->opts->support_matching_rule = true;
+    } else if (ret == EIO) {
+        /* The search failed. Disable support for
+         * matching rule lookups.
+         */
+        state->opts->support_matching_rule = false;
+    } else {
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              ("Unexpected error while testing for matching rule support\n"));
+        tevent_req_error(req, ret);
+        return;
+    }
+
+    DEBUG(SSSDBG_CONF_SETTINGS,
+          ("LDAP server %s the matching rule extension\n",
+          state->opts->support_matching_rule
+              ? "supports"
+              : "does not support"));
 
     tevent_req_done(req);
 }
@@ -1156,8 +1239,10 @@ static errno_t sdap_get_generic_ext_step(struct tevent_req *req)
      */
     talloc_zfree(state->op);
 
-    DEBUG(6, ("calling ldap_search_ext with [%s][%s].\n", state->filter,
-                                                          state->search_base));
+    DEBUG(SSSDBG_TRACE_FUNC,
+         ("calling ldap_search_ext with [%s][%s].\n",
+          state->filter ? state->filter : "no filter",
+          state->search_base));
     if (DEBUG_IS_SET(SSSDBG_TRACE_LIBS)) {
         int i;
 
@@ -1291,6 +1376,18 @@ static void sdap_get_generic_ext_done(struct sdap_op *op,
             /* Try to return what we've got */
             DEBUG(SSSDBG_MINOR_FAILURE,
                   ("LDAP sizelimit was exceeded, returning incomplete data\n"));
+        } else if (result == LDAP_INAPPROPRIATE_MATCHING) {
+            /* This error should only occur when we're testing for
+             * specialized functionality like the ldap matching rule
+             * filter for Active Directory. Warn at a higher log
+             * level and return EIO.
+             */
+            DEBUG(SSSDBG_TRACE_INTERNAL,
+                  ("LDAP_INAPPROPRIATE_MATCHING:  %s\n",
+                   errmsg ? errmsg : "no errmsg set"));
+            ldap_memfree(errmsg);
+            tevent_req_error(req, EIO);
+            return;
         } else if (result != LDAP_SUCCESS && result != LDAP_NO_SUCH_OBJECT) {
             DEBUG(SSSDBG_OP_FAILURE,
                   ("Unexpected result from ldap: %s(%d), %s\n",
@@ -1825,6 +1922,8 @@ static errno_t sdap_asq_search_parse_entry(struct sdap_handle *sh,
     struct sdap_attr_map *map;
     int num_attrs;
     struct sdap_deref_attrs **res;
+    char *tmp;
+    char *dn;
     TALLOC_CTX *tmp_ctx;
 
     tmp_ctx = talloc_new(NULL);
@@ -1847,6 +1946,20 @@ static errno_t sdap_asq_search_parse_entry(struct sdap_handle *sh,
         res[mi]->attrs = NULL;
     }
 
+
+    tmp = ldap_get_dn(sh->ldap, msg->msg);
+    if (!tmp) {
+        ret = EINVAL;
+        goto done;
+    }
+
+    dn = talloc_strdup(tmp_ctx, tmp);
+    ldap_memfree(tmp);
+    if (!dn) {
+        ret = ENOMEM;
+        goto done;
+    }
+
     /* Find all suitable maps in the list */
     vals = ldap_get_values_len(sh->ldap, msg->msg, "objectClass");
     for (mi =0; mi < state->num_maps; mi++) {
@@ -1856,12 +1969,20 @@ static errno_t sdap_asq_search_parse_entry(struct sdap_handle *sh,
             if (strncasecmp(state->maps[mi].map[0].name,
                             vals[i]->bv_val, vals[i]->bv_len) == 0) {
                 /* it's an entry of the right type */
+                DEBUG(SSSDBG_TRACE_INTERNAL,
+                      ("Matched objectclass [%s] on DN [%s], will use associated map\n",
+                       state->maps[mi].map[0].name, dn));
                 map = state->maps[mi].map;
                 num_attrs = state->maps[mi].num_attrs;
                 break;
             }
         }
-        if (!map) continue;
+        if (!map) {
+            DEBUG(SSSDBG_TRACE_INTERNAL,
+                  ("DN [%s] did not match the objectClass [%s]\n",
+                   dn, state->maps[mi].map[0].name));
+            continue;
+        }
 
         ret = sdap_parse_entry(res[mi], sh, msg,
                                map, num_attrs,
@@ -1870,7 +1991,6 @@ static errno_t sdap_asq_search_parse_entry(struct sdap_handle *sh,
             DEBUG(3, ("sdap_parse_entry failed [%d]: %s\n", ret, strerror(ret)));
             goto done;
         }
-
     }
     ldap_value_free_len(vals);
 
@@ -2076,114 +2196,6 @@ bool sdap_has_deref_support(struct sdap_handle *sh, struct sdap_options *opts)
     return false;
 }
 
-errno_t sdap_check_aliases(struct sysdb_ctx *sysdb,
-                           struct sysdb_attrs *user_attrs,
-                           struct sss_domain_info *dom,
-                           struct sdap_options *opts,
-                           bool steal_memberships)
-{
-    errno_t ret;
-    const char **aliases = NULL;
-    const char *name = NULL;
-    struct ldb_message *msg;
-    TALLOC_CTX *tmp_ctx = NULL;
-    char **parents;
-    uid_t alias_uid, uid;
-    int i;
-
-    tmp_ctx = talloc_new(NULL);
-    if (!tmp_ctx) return ENOMEM;
-
-    ret = sysdb_attrs_primary_name(sysdb, user_attrs,
-                                   opts->user_map[SDAP_AT_USER_NAME].sys_name,
-                                   &name);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_TRACE_INTERNAL, ("Could not get the primary name\n"));
-        goto done;
-    }
-
-    ret = sysdb_attrs_get_uint32_t(user_attrs,
-                                   opts->user_map[SDAP_AT_USER_UID].sys_name,
-                                   &uid);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_TRACE_INTERNAL, ("Could not get UID\n"));
-        goto done;
-    }
-
-    ret = sysdb_attrs_get_aliases(tmp_ctx, user_attrs, name,
-                                  !dom->case_sensitive, &aliases);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_TRACE_INTERNAL, ("Failed to get the alias list\n"));
-        goto done;
-    }
-
-    for (i = 0; aliases[i]; i++) {
-        /* In RFC2307 schema, another group might be referencing user
-         * using secondary name, so there might be fake users in the cache
-         * from a previous getgr call */
-        ret = sysdb_search_user_by_name(tmp_ctx, sysdb,
-                                        aliases[i], NULL, &msg);
-        if (ret && ret != ENOENT) {
-            DEBUG(SSSDBG_TRACE_INTERNAL, ("Error searching the cache\n"));
-            goto done;
-        } else if (ret == ENOENT) {
-            DEBUG(SSSDBG_TRACE_INTERNAL,
-                  ("No user with primary name same as alias %s\n", aliases[i]));
-            continue;
-        }
-
-        alias_uid = ldb_msg_find_attr_as_uint64(msg, SYSDB_UIDNUM, 0);
-        if (alias_uid) {
-            if (alias_uid == uid) {
-                DEBUG(SSSDBG_TRACE_INTERNAL,
-                      ("User already cached, skipping\n"));
-                continue;
-            }
-            DEBUG(SSSDBG_FATAL_FAILURE,
-                  ("Cache contains non-fake user with same name "
-                   "as alias %s\n", aliases[i]));
-            ret = EIO;
-            goto done;
-        }
-        DEBUG(SSSDBG_TRACE_FUNC, ("%s is a fake user\n", aliases[i]));
-
-        if (steal_memberships) {
-            /* Get direct sysdb parents */
-            ret = sysdb_get_direct_parents(tmp_ctx, sysdb, dom,
-                                           SYSDB_MEMBER_USER,
-                                           aliases[i], &parents);
-            if (ret) {
-                DEBUG(SSSDBG_FATAL_FAILURE,
-                      ("Could not get direct parents for %s: %d [%s]\n",
-                       aliases[i], ret, strerror(ret)));
-                goto done;
-            }
-
-            ret = sysdb_update_members(sysdb, name, SYSDB_MEMBER_USER,
-                                    (const char *const *) parents,
-                                    NULL);
-            if (ret != EOK) {
-                DEBUG(SSSDBG_FATAL_FAILURE,
-                      ("Membership update failed [%d]: %s\n",
-                      ret, strerror(ret)));
-                goto done;
-            }
-        }
-
-        ret = sysdb_delete_user(sysdb, aliases[i], alias_uid);
-        if (ret) {
-            DEBUG(SSSDBG_FATAL_FAILURE,
-                  ("Error deleting fake user %s\n", aliases[i]));
-            goto done;
-        }
-    }
-
-    ret = EOK;
-done:
-    talloc_free(tmp_ctx);
-    return ret;
-}
-
 errno_t
 sdap_attrs_add_ldap_attr(struct sysdb_attrs *ldap_attrs,
                          const char *attr_name,
