@@ -31,6 +31,8 @@
 #include "providers/ldap/sdap_id_op.h"
 #include "providers/fail_over.h"
 
+#define AD_TOKENGROUPS_ATTR "tokenGroups"
+
 struct tevent_req *sdap_connect_send(TALLOC_CTX *memctx,
                                      struct tevent_context *ev,
                                      struct sdap_options *opts,
@@ -195,12 +197,6 @@ int sdap_deref_search_recv(struct tevent_req *req,
                            size_t *reply_count,
                            struct sdap_deref_attrs ***reply);
 
-errno_t sdap_check_aliases(struct sysdb_ctx *sysdb,
-                           struct sysdb_attrs *user_attrs,
-                           struct sss_domain_info *dom,
-                           struct sdap_options *opts,
-                           bool steal_memberships);
-
 errno_t
 sdap_attrs_add_ldap_attr(struct sysdb_attrs *ldap_attrs,
                          const char *attr_name,
@@ -249,4 +245,50 @@ enum_services_send(TALLOC_CTX *memctx,
 errno_t
 enum_services_recv(struct tevent_req *req);
 
+/* OID documented in
+ * http://msdn.microsoft.com/en-us/library/windows/desktop/aa746475%28v=vs.85%29.aspx
+ */
+#define SDAP_MATCHING_RULE_IN_CHAIN "1.2.840.113556.1.4.1941"
+
+struct tevent_req *
+sdap_get_ad_match_rule_members_send(TALLOC_CTX *mem_ctx,
+                                    struct tevent_context *ev,
+                                    struct sdap_options *opts,
+                                    struct sdap_handle *sh,
+                                    struct sysdb_attrs *group,
+                                    int timeout);
+
+errno_t
+sdap_get_ad_match_rule_members_recv(struct tevent_req *req,
+                                    TALLOC_CTX *mem_ctx,
+                                    size_t *num_users,
+                                    struct sysdb_attrs ***users);
+
+struct tevent_req *
+sdap_get_ad_match_rule_initgroups_send(TALLOC_CTX *mem_ctx,
+                                       struct tevent_context *ev,
+                                       struct sdap_options *opts,
+                                       struct sysdb_ctx *sysdb,
+                                       struct sdap_handle *sh,
+                                       const char *name,
+                                       const char *orig_dn,
+                                       int timeout);
+
+errno_t
+sdap_get_ad_match_rule_initgroups_recv(struct tevent_req *req);
+
+
+struct tevent_req *
+sdap_get_ad_tokengroups_initgroups_send(TALLOC_CTX *mem_ctx,
+                                        struct tevent_context *ev,
+                                        struct sdap_options *opts,
+                                        struct sysdb_ctx *sysdb,
+                                        struct sdap_handle *sh,
+                                        const char *name,
+                                        const char *orig_dn,
+                                        int timeout);
+
+errno_t
+sdap_get_ad_tokengroups_initgroups_recv(struct tevent_req *req);
+
 #endif /* _SDAP_ASYNC_H_ */
