@@ -160,8 +160,11 @@ int sdap_save_user(TALLOC_CTX *memctx,
         /* Store the UID in the ldap_attrs so it doesn't get
          * treated as a missing attribute from LDAP and removed.
          */
-        ret = sysdb_attrs_add_uint32(attrs, SYSDB_UIDNUM, uid);
-        if (ret != EOK) goto done;
+        ret = sdap_replace_id(attrs, SYSDB_UIDNUM, uid);
+        if (ret) {
+            DEBUG(SSSDBG_OP_FAILURE, ("Cannot set the id-mapped UID\n"));
+            goto done;
+        }
     } else {
         ret = sysdb_attrs_get_uint32_t(attrs,
                                        opts->user_map[SDAP_AT_USER_UID].sys_name,
