@@ -90,6 +90,34 @@ autofs_setent_notify(struct autofs_map_ctx *map_ctx, errno_t ret)
     setent_notify(&map_ctx->reqs, ret);
 }
 
+errno_t
+autofs_orphan_maps(struct autofs_ctx *actx)
+{
+    int hret;
+    unsigned long mcount;
+    unsigned long i;
+    hash_key_t *maps;
+
+    if (!actx || !actx->maps) {
+        return EINVAL;
+    }
+
+    hret = hash_keys(actx->maps, &mcount, &maps);
+    if (hret != HASH_SUCCESS) {
+        return EIO;
+    }
+
+    for (i = 0; i < mcount; i++) {
+        hret = hash_delete(actx->maps, &maps[i]);
+        if (hret != HASH_SUCCESS) {
+            DEBUG(SSSDBG_MINOR_FAILURE, ("Could not delete key from hash\n"));
+            continue;
+        }
+    }
+
+    return EOK;
+}
+
 static errno_t
 get_autofs_map(struct autofs_ctx *actx,
                char *mapname,
@@ -118,6 +146,27 @@ get_autofs_map(struct autofs_ctx *actx,
 
 static int autofs_map_hash_remove (TALLOC_CTX *ctx);
 
+void
+autofs_map_hash_delete_cb(hash_entry_t *item,
+                          hash_destroy_enum deltype, void *pvt)
+{
+    struct autofs_map_ctx *map;
+
+    if (deltype != HASH_ENTRY_DESTROY) {
+        return;
+    }
+
+    map = talloc_get_type(item->value.ptr, struct autofs_map_ctx);
+    if (!map) {
+        DEBUG(SSSDBG_OP_FAILURE, ("Invalid autofs map\n"));
+        return;
+    }
+
+    /* So that the destructor wouldn't attempt to remove the map from hash
+     * table */
+    map->map_table = NULL;
+}
+
 static errno_t
 set_autofs_map(struct autofs_ctx *actx,
                struct autofs_map_ctx *map)
@@ -158,6 +207,12 @@ autofs_map_hash_remove(TALLOC_CTX *ctx)
     struct autofs_map_ctx *map =
             talloc_get_type(ctx, struct autofs_map_ctx);
 
+    if (map->map_table == NULL) {
+        DEBUG(SSSDBG_TRACE_LIBS, ("autofs map [%s] was already removed\n",
+              map->mapname));
+        return 0;
+    }
+
     key.type = HASH_KEY_STRING;
     key.str = map->mapname;
 
@@ -342,6 +397,11 @@ set_autofs_map_lifetime(uint32_t lifetime,
     }
 }
 
+static errno_t
+setautomntent_get_autofs_map(struct autofs_ctx *actx,
+                             char *mapname,
+                             struct autofs_map_ctx **map);
+
 static struct tevent_req *
 setautomntent_send(TALLOC_CTX *mem_ctx,
                    const char *rawname,
@@ -415,7 +475,7 @@ setautomntent_send(TALLOC_CTX *mem_ctx,
     /* Is the result context already available?
      * Check for existing lookups for this map
      */
-    ret = get_autofs_map(actx, state->mapname, &state->map);
+    ret = setautomntent_get_autofs_map(actx, state->mapname, &state->map);
     if (ret == EOK) {
         /* Another process already requested this map
          * Check whether it's ready for processing.
@@ -531,6 +591,25 @@ fail:
     return req;
 }
 
+static errno_t
+setautomntent_get_autofs_map(struct autofs_ctx *actx,
+                             char *mapname,
+                             struct autofs_map_ctx **map)
+{
+    errno_t ret;
+
+    if (strcmp(mapname, "auto.master") == 0) {
+        /* Iterate over the hash and remove all maps */
+        ret = autofs_orphan_maps(actx);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_MINOR_FAILURE, ("Could not remove existing maps from hash\n"));
+        }
+        return ENOENT;
+    }
+
+    return get_autofs_map(actx, mapname, map);
+}
+
 static errno_t
 lookup_automntmap_update_cache(struct setautomntent_lookup_ctx *lookup_ctx);
 
@@ -691,8 +770,10 @@ lookup_automntmap_update_cache(struct setautomntent_lookup_ctx *lookup_ctx)
     struct dp_callback_ctx *cb_ctx = NULL;
 
     if (dctx->map != NULL) {
-        cache_expire = ldb_msg_find_attr_as_uint64(dctx->map,
-                                                   SYSDB_CACHE_EXPIRE, 0);
+        if (strcmp(lookup_ctx->mapname, "auto.master") != 0) {
+            cache_expire = ldb_msg_find_attr_as_uint64(dctx->map,
+                                                       SYSDB_CACHE_EXPIRE, 0);
+        }
 
         /* if we have any reply let's check cache validity */
         ret = sss_cmd_check_cache(dctx->map, 0, cache_expire);
@@ -859,7 +940,7 @@ sss_autofs_cmd_getautomntent(struct cli_ctx *client)
 
     SAFEALIGN_COPY_UINT32_CHECK(&namelen, body+c, blen, &c);
 
-    if (namelen == 0) {
+    if (namelen == 0 || namelen > blen - c) {
         ret = EINVAL;
         goto done;
     }
@@ -1134,7 +1215,7 @@ sss_autofs_cmd_getautomntbyname(struct cli_ctx *client)
     /* FIXME - split out a function to get string from <len><str>\0 */
     SAFEALIGN_COPY_UINT32_CHECK(&namelen, body+c, blen, &c);
 
-    if (namelen == 0) {
+    if (namelen == 0 || namelen > blen - c) {
         ret = EINVAL;
         goto done;
     }
@@ -1158,7 +1239,7 @@ sss_autofs_cmd_getautomntbyname(struct cli_ctx *client)
     /* FIXME - split out a function to get string from <len><str>\0 */
     SAFEALIGN_COPY_UINT32_CHECK(&keylen, body+c, blen, &c);
 
-    if (keylen == 0) {
+    if (keylen == 0 || keylen > blen - c) {
         ret = EINVAL;
         goto done;
     }
