@@ -538,7 +538,7 @@ static int sss_dp_init(struct resp_ctx *rctx,
     ret = sbus_client_init(rctx, rctx->ev,
                            be_conn->sbus_address,
                            intf, &be_conn->conn,
-                           NULL, NULL);
+                           NULL, rctx);
     if (ret != EOK) {
         DEBUG(0, ("Failed to connect to monitor services.\n"));
         return ret;
@@ -839,6 +839,7 @@ int sss_process_init(TALLOC_CTX *mem_ctx,
 
     ret = sysdb_init(rctx, cdb, NULL, false, &rctx->db_list);
     if (ret != EOK) {
+        SYSDB_VERSION_ERROR_DAEMON(ret);
         DEBUG(0, ("fatal error initializing resp_ctx\n"));
         return ret;
     }
