@@ -236,12 +236,11 @@ int LOCAL_pam_handler(struct pam_auth_req *preq)
         return ENOMEM;
     }
 
-    ret = sysdb_get_ctx_from_list(preq->cctx->rctx->db_list,
-                                  preq->domain, &lreq->dbctx);
-    if (ret != EOK) {
+    lreq->dbctx = preq->domain->sysdb;
+    if (lreq->dbctx == NULL) {
         DEBUG(0, ("Fatal: Sysdb CTX not found for this domain!\n"));
         talloc_free(lreq);
-        return ret;
+        return ENOENT;
     }
     lreq->ev = preq->cctx->ev;
     lreq->preq = preq;
