@@ -19,6 +19,7 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include <nss.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <talloc.h>
@@ -106,6 +107,19 @@ int main(int argc, const char **argv)
 
     /* groupdel */
     ret = groupdel(tctx, tctx->sysdb, tctx->octx);
+    if (ret != EOK) {
+        goto done;
+    }
+
+    /* Delete group from memory cache */
+    ret = sss_mc_refresh_group(pc_groupname);
+    if (ret != EOK) {
+        ERROR("NSS request failed (%1$d). Entry might remain in memory "
+              "cache.\n", ret);
+        /* Nothing we can do about it */
+    }
+
+    ret = EOK;
 
 done:
     if (ret) {
