@@ -467,7 +467,8 @@ int useradd(TALLOC_CTX *mem_ctx,
     int ret;
 
     ret = sysdb_add_user(sysdb, data->name, data->uid, data->gid,
-                         data->gecos, data->home, data->shell, NULL, 0, 0);
+                         data->gecos, data->home, data->shell,
+                         NULL, NULL, 0, 0);
     if (ret) {
         goto done;
     }
