@@ -29,6 +29,14 @@
 
 #define LDAP_X_SSSD_PASSWORD_EXPIRED 0x555D
 
+#ifndef LDAP_CONTROL_PWEXPIRED
+#define LDAP_CONTROL_PWEXPIRED "2.16.840.1.113730.3.4.4"
+#endif
+
+#ifndef LDAP_CONTROL_PWEXPIRING
+#define LDAP_CONTROL_PWEXPIRING "2.16.840.1.113730.3.4.5"
+#endif
+
 #ifdef LDAP_OPT_DIAGNOSTIC_MESSAGE
 #define SDAP_DIAGNOSTIC_MESSAGE LDAP_OPT_DIAGNOSTIC_MESSAGE
 #else
