@@ -41,19 +41,9 @@ sss_ssh_make_ent(TALLOC_CTX *mem_ctx,
                  struct ldb_message *msg,
                  struct sss_ssh_ent **result);
 
-char *
-sss_ssh_get_pubkey_algorithm(TALLOC_CTX *mem_ctx,
-                             struct sss_ssh_pubkey *pubkey);
-
-enum sss_ssh_pubkey_format {
-    SSS_SSH_FORMAT_RAW,
-    SSS_SSH_FORMAT_OPENSSH
-};
-
-char *
+errno_t
 sss_ssh_format_pubkey(TALLOC_CTX *mem_ctx,
-                      struct sss_ssh_ent *ent,
                       struct sss_ssh_pubkey *pubkey,
-                      enum sss_ssh_pubkey_format format);
+                      char **result);
 
 #endif /* _SSS_SSH_H_ */
